+++
+++  Adjoint functors  Left -| Right
+++
+++  << and >> are the left and right adjunctions: they are the defining isomorphism
+++
+++    Hom(Left A,B) << >> Hom( A, Right B)
+++
+++  unit and counit are the equivalent natural transformations.  left and right are the 
+++  action of the Left and Right functors on the arrows.  unit, counit, left and right are 
+++  provided by default.
+++
define Adjoint(ObjA:Category,ObjB:Category, Left:ObjA->ObjB, Right:ObjB->ObjA):Category == with  {
    >>: (A:ObjA,B:ObjB,  Left A ->       B ) -> (      A -> Right B );  -- 1/2 of the adj. isomorphism
    <<: (A:ObjA,B:ObjB,       A -> Right B ) -> ( Left A ->       B );  -- 2/2 of the adj. isomorphism
    
    unit:   (A:ObjA) -> (           A -> Right  Left A);    --   unit natural transformation
    counit: (B:ObjB) -> (Left Right B ->             B);    -- counit natural transformation
    
    left:  (X:ObjA,Y:ObjA,X->Y) -> (  Left X ->  Left Y );  --  Left functor action on morphisms
    right: (X:ObjB,Y:ObjB,X->Y) -> ( Right X -> Right Y );  -- Right functor action on morphisms
    default  {
        unit  (A:ObjA):(A -> Right Left A)               == >>(      A,Left A,(l:Left  A):Left  A +-> l);
        counit(B:ObjB):(Left Right B -> B)               == <<(Right B,     B,(r:Right B):Right B +-> r);
        left (X:ObjA,Y:ObjA,f:X->Y):( Left X -> Left  Y) == <<(      X,Left Y,(x:           X):Right Left Y +->     unit(Y)(f x));
        right(X:ObjB,Y:ObjB,f:X->Y):(Right X -> Right Y) == >>(Right X,     Y,(x:Left Right X):           Y +-> f counit(X)(x) )
    }
}
+++
+++  The right adjoint of Left
+++
define RightAdjoint(ObjA:Category,ObjB:Category,Left:ObjA->ObjB):Category == with {
    Right: ObjB -> ObjA;
    Adjoint(ObjA,ObjB,Left,Right)
}
+++
+++  The left adjoint of Right
+++    
define LeftAdjoint(ObjA:Category,ObjB:Category,Right:ObjB->ObjA):Category == with {
    Left:  ObjA -> ObjB;
    Adjoint(ObjA,ObjB,Left,Right)
}
+++
+++  Free construction
+++
define FreeConstruction(ObjA:Category,ObjB:Category,Forget:ObjB->ObjA):Category == LeftAdjoint(ObjA,ObjB,Forget) with 

--LeftCategory(ObjA:Category,CatA:MathCategory ObjA, ObjB:Category,  Left:ObjA->ObjB,  Ad:RightAdjoint(ObjA,ObjB,Left)):_
--     MathCategory ObjB with {
--         if   CatA    has      Initial ObjA  then       Initial ObjB;
--         if   CatA    has    CoProduct ObjA  then     CoProduct ObjB;
--         if   CatA    has  CoEqualizer ObjA  then   CoEqualizer ObjB;
--         if   CatA    has      Pushout ObjA  then       Pushout ObjB;
--} == add {
--    if CatA has Initial ObjA then {
--        Zero():ObjB == {
--            LC0:ObjB == Left Zero() add;
--            LC0 add
--        }
--        zero(A:ObjB):(Zero()->A) == {
--            A1:ObjA == (Right$Ad) A;
--            z:(Zero$CatA)()->A1 == (zero$CatA)(A1);
--            
--            error " "
--        }
--    }
--    if CatA has CoProduct ObjA then {
--        CoProduct(A:ObjB,B:ObjB):(AB:ObjB,A->AB,B->AB,(X:ObjB)->(A->X,B->X)->(AB->X)) == {
--            A1:ObjA == (Right$Ad) A;
--            B1:ObjA == (Right$Ad) B;
--            (AB1:ObjA,ia1:A1->AB1,ib1:B1->AB1,coproduct1:(X1:ObjA)->(A1->X1,B1->X1)->(AB1->X1)) == (CoProduct$CatA)(A1,B1);
--            error " "
--        }
--    }
--}

--define LeftCategory(ObjA:Category,CatA:MathCategory ObjA,  ObjB:Category,Left:ObjA->ObjB, _
--          Ad:RightAdjoint(ObjA,ObjB,Left)):Category == with {
--    if   CatA    has      Initial ObjA  then     Initial ObjB;
--    if   CatA    has    CoProduct ObjA  then   CoProduct ObjB;
--    if   CatA    has  CoEqualizer ObjA  then CoEqualizer ObjB;
--    if   CatA    has      Pushout ObjA  then     Pushout ObjB;
--}
--default {
--    if CatA has Initial ObjA then {
--        Zero():ObjB == {
--            LC0:ObjB == Left Zero() add;
--            LC0 add
--        }
--    }
--}

-------  this won't work due to bugs in 1.1.12p6
--+++
--+++  Adjoint in general from ObjA to ObjB
--+++
--define Adjoint(ObjA:Category,ObjB:Category):Category == with
--    Left:  ObjA -> ObjB
--    Right: ObjB -> ObjA
--    Adjoint(ObjA,ObjB,Left,Right)
--
--+++
--+++  Free construction via a forgetful functor.  This is an adjoint with
--+++  conventional renaming.
--+++
--define FreeConstruction(ObjA:Category,ObjB:Category):Category == with
--    Free:   ObjA -> ObjB
--    Free:   (X:ObjA,Y:ObjA,X->Y) -> (Free X -> Free Y)
--    Forget: ObjB -> ObjA 
--    Forget: (X:ObjB,Y:ObjB,X->Y) -> (Forget X -> Forget Y)
--    Adjoint(ObjA,ObjB,Free,Forget)
--    default
--        Free  (X:Domain,Y:Domain,f:X->Y):(Free X->Free Y) == left(X,Y,f)
--        Forget(X:Set,Y:Set,f:X->Y):(Forget X->Forget Y) == right(X,Y,f)


