
#include "base.as"

#library lPreorders    "Preorders.ao"

import from lPreorders

+++
+++  The category of Lattices.  A lattice is a Preorder with least upper bound (/\), 
+++  greatest lower bound (\/) minimum (min) and maximum (max) elements.
+++
define Lattice:Category == Preorder with 
    /\: (%,%) -> %
    \/: (%,%) -> %
   
+++
+++  The Forgetful functor to Preorder
+++
ForgetToPreorder(L:Lattice):Preorder == L add

+++
+++  The Dual Functor taking a lattice to it's dual in the partial order sense
+++
DualLattice(L:Lattice):Lattice ==
    W0:Lattice == (Dual$lPreorders)(L) add
        Rep == L; import from Rep
	(x:%)/\(y:%):% == per ( (rep x)\/(rep y) )
	(x:%)\/(y:%):% == per ( (rep x)/\(rep y) )
    W0 add
        
+++
+++  The Interval functor
+++
Interval(L:Lattice):Lattice == 
    W4:Lattice == (Interval$lPreorders)(L) add
        Rep == Record(a:L,b:L); import from Rep
        (x:%)/\(y:%):% == per [ (rep x).a \/ (rep y).a, (rep x).b /\ (rep y).b ]
        (x:%)\/(y:%):% == per [ (rep x).a /\ (rep y).a, (rep x).b \/ (rep y).b ]
    W4 add

+++
+++  The Category of Lattices.
+++    
LatticeCategory:MathCategory Lattice _
    with             Initial Lattice _
    with             Product Lattice _
--    with         Exponential Lattice
== add
    Zero():Lattice == 
        W1:Lattice == (Zero()$PreorderCategory) add
	    (x:%)/\(y:%):% == never
	    (x:%)\/(y:%):% == never
	W1 add
    zero(A:Lattice):(Zero()->A) == (z:Zero()):A +-> never
    
    One():Lattice == 
        W2:Lattice == (One()$PreorderCategory) add
	    Rep == SingleInteger; import from Rep
	    (x:%)/\(y:%):% == per 1
	    (x:%)\/(y:%):% == per 1
	W2 add
    one(A:Lattice):(A->One()) == (a:A):One() +-> (1$SingleInteger) pretend One()

    Product(A:Lattice,B:Lattice):(AB:Lattice,AB->A,AB->B,(X:Lattice)->(X->A,X->B)->(X->AB)) == 
        (P:Preorder,pa:P->A,pb:P->B,product:(X:Preorder)->(X->A,X->B)->(X->P)) == (Product$PreorderCategory)(A,B)
	extend P: with Lattice == P add
	    Rep == Record(a:A,b:B); import from Rep
	    import from A,B
	    (x:%)/\(y:%):% == per [ (rep x).a /\ (rep y).a , (rep x).b /\ (rep y).b ]
	    (x:%)\/(y:%):% == per [ (rep x).a \/ (rep y).a , (rep x).b \/ (rep y).b ]
	(P,pa,pb,product) 
	
    (B:Lattice)^(A:Lattice):Lattice == 
        W3:Lattice == add
            Rep ==> A->B
	    (f:%)=(g:%):Boolean == error "equality not available for Lattice morphisms"
	    (f:%)<=(g:%):Boolean == error "<= not available for Lattice morphisms."
	    (f:%)/\(g:%):% == per ( (a:A):B +-> (rep f)(a)/\(rep g)(a) )
	    (f:%)\/(g:%):% == per ( (a:A):B +-> (rep f)(a)\/(rep g)(a) )
            (t:TextWriter)<<(x:%):TextWriter == t << "[Exponential Object]"
        W3 add	

